;==================================================================================
; Contents of this file are copyright Grant Searle
;
; Modified for Zeus.
;
; You have permission to use this for NON COMMERCIAL USE ONLY
; If you wish to use it elsewhere, please include an acknowledgement to myself.
;
; http://searle.hostei.com/grant/index.html
;
; eMail: home.micros01@btinternet.com
;
; If the above don't work, please perform an Internet search to see if I have
; updated the web page hosting service.
;
;==================================================================================

loadAddr        equ    $D000
numSecs         equ    24      ; Number of 512 sectors to be loaded

; CF registers
CF_DATA         equ    $10
CF_FEATURES     equ    $11
CF_ERROR        equ    $11
CF_SECCOUNT     equ    $12
CF_SECTOR       equ    $13
;CF_CYL_LOW      equ    $14
;CF_CYL_HI       equ    $15
;CF_HEAD         equ    $16
CF_STATUS       equ    $17
CF_COMMAND      equ    $17

CF_LBA0         equ    $13
CF_LBA1         equ    $14
CF_LBA2         equ    $15
CF_LBA3         equ    $16

;CF Features
CF_8BIT         equ    1
CF_NOCACHE      equ    $82
;CF Commands
CF_READ_SEC     equ    $20
CF_WRITE_SEC    equ    $30
CF_SET_FEAT     equ    $EF

LF              equ    $0A             ;line feed
FF              equ    $0C             ;form feed
CR              equ    $0D             ;carriage RETurn

;================================================================================================

                org    $5000           ; Loader origin.

                call    printInline
                db "CP/M System Transfer by G. Searle 2012"
                db CR,LF,0

                call    cfWait
                ld      a,CF_8BIT       ; Set IDE to be 8bit
                out     (CF_FEATURES),a
                ld      a,CF_SET_FEAT
                out     (CF_COMMAND),a

                call    cfWait
                ld      a,CF_NOCACHE    ; No write cache
                out     (CF_FEATURES),a
                ld      a,CF_SET_FEAT
                out     (CF_COMMAND),a

                ld      b,numSecs

                ld      a,0
                ld      (secNo),a
                ld      hl,loadAddr
                ld      (dmaAddr),hl
processSectors:

                call    cfWait

                ld      a,(secNo)
                out     (CF_LBA0),a
                ld      a,0
                out     (CF_LBA1),a
                out     (CF_LBA2),a
                ld      a,$E0
                out     (CF_LBA3),a
                ld      a,1
                out     (CF_SECCOUNT),a

                call    write

                ld      de,$0200
                ld      hl,(dmaAddr)
                add     hl,de
                ld      (dmaAddr),hl
                ld      a,(secNo)
                inc     a
                ld      (secNo),a

                djnz    processSectors

                call    printInline
                db CR,LF
                db "System transfer complete"
                db CR,LF,0

                ret


;================================================================================================
; Write physical sector to host
;================================================================================================

write:
                push    af
                push    bc
                push    hl


                call    cfWait

                ld      a,CF_WRITE_SEC
                out     (CF_COMMAND),a

                call    cfWait

                ld      c,4
                ld      hl,(dmaAddr)
wr4secs:
                ld      b,128
wrByte:         ld      a,(hl)
                nop
                nop
                out     (CF_DATA),a
                inc     hl
                dec     b
                jr      nz, wrByte

                dec     c
                jr      nz,wr4secs

                pop     hl
                pop     bc
                pop     af

                ret

;================================================================================================
; Wait for disk to be ready (busy=0,ready=1)
;================================================================================================
cfWait:
                push    af
cfWait1:
                in      a,(CF_STATUS)
                and     $80
                cp      $80
                jr      z,cfWait1
                pop     af
                ret


;================================================================================================
; Utilities
;================================================================================================

printInline:
                ex      (sp),hl         ; PUSH HL and put RET ADDress into HL
                push    af
                push    bc
nextILChar:     ld      a,(hl)
                cp      0
                jr      z,endOfPrint
                rst     $08
                inc     hl
                jr      nextILChar
endOfPrint:     inc     hl              ; Get past "null" terminator
                pop     bc
                pop     af
                ex      (sp),hl         ; PUSH new RET ADDress on stack and restore HL
                ret

dmaAddr         dw     0
secNo           db     0

; May as well load CP/M into memory while we're at it, so it's there to put onto the flash

                import_bin "cpm22.bin",$D000
                import_bin "cbios64.bin",$E600

